{ Office 97 Controls version 1.1, copyright  A. Meeder, 1996

  ** It's freeware, so you don't have to pay me, but I do not guarantee the working of it

  The idea came from the personal assistent of MS Office 97. To use these components:

  Special properties of TOfficeButton:
  * BorderColor, color of the border when the button has the mouse
  * BorderWidth, basic width of the border (1 .. 3)
  * RoundSize, used to change the corners of the button

  Sepcial properties of TOfficeLabel:
  * LEDType, (RoundLED, TriLEDUp, TriLEDDown)

  If you have suggestions about TOButton or make changes, please mail me: ameeder@dds.nl
}

unit OControls;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type

//-----------------------------------------------------------------------------------
// Start of TOfficeButton defenition

  TBorderWidth = 1..3;

  TOfficeButton = class(TGraphicControl)
  private
    { Private declarations }
    FRoundSize: integer;
    FCaption: TCaption;
    FGlyph: TBitmap;
    FBorderColor: TColor;
    FBorderWidth: TBorderWidth;
    FFont: TFont;

    HasMouse: boolean;
    IsMouseDown: boolean;
    ExtraSpace: word;

    procedure SetBorderColor(value: TColor);
    procedure SetBorderWidth(value: TBorderWidth);
    procedure SetCaption(value: TCaption);
    procedure SetFont(value: TFont);
    procedure SetRoundSize(value: integer);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;  // New event
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;  // New event
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Caption: TCaption read FCaption write SetCaption;
    property BorderColor: TColor read FBorderColor write FBorderColor default clGray;
    property BorderWidth: TBorderWidth read FBorderWidth write SetBorderWidth default 1;
    property Font: TFont read FFont write SetFont;
    property RoundSize: integer read FRoundSize write SetRoundSize default 15;
    property Enabled;
    property ShowHint;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

//-----------------------------------------------------------------------------------
// Start of TOfficeLabel defenition

  TLEDType = (RoundLED, TriLEDDown, TriLEDUp);

  TOfficeLabel = class(TGraphicControl)
  private
    { Private declarations }
    FCaption: TCaption;
    FLedType: TLEDType;
    FGlyph: TBitmap;
    FFont: TFont;

    HasMouse, IsMouseDown: boolean;

    procedure SetCaption(value: TCaption);
    procedure SetLEDType(value: TLEDType);
    procedure SetFont(value: TFont);
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  protected
    { Protected declarations }
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure Paint; override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property Caption: TCaption read FCaption write SetCaption;
    property LEDType: TLEDType read FLEDType write SetLEDType default RoundLED;
    property Font: TFont read FFont write SetFont;
    // property Color;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

// Recourse with LED-bitmaps
{$R OFFICECONTROLS.RES}

procedure TOfficeButton.SetBorderColor(value: TColor);
begin
  if Value <> FBorderColor then
  begin
    FBorderColor := Value;
    Invalidate;
  end;
end;

procedure TOfficeButton.SetBorderWidth(value: TBorderWidth);
begin
  if Value <> FBorderWidth then
  begin
    FBorderWidth := Value;
    Invalidate;
  end;
end;

procedure TOfficeButton.SetCaption(value: TCaption);
begin
  if Value <> FCaption then
  begin
    FCaption := Value;
    Invalidate;
  end;
end;

procedure TOfficeButton.SetFont(value: TFont);
begin
  if Value <> FFont then
  begin
    FFont.Assign(Value);
    Invalidate;
  end;
end;

procedure TOfficeButton.SetRoundSize(value: integer);
begin
  if Value <> FRoundSize then
  begin
    FRoundSize := Value;
    Invalidate;
  end;
end;

procedure TOfficeButton.CMMouseEnter(var Message: TMessage);
begin
  HasMouse := true;
  if IsMouseDown then ExtraSpace := 1 else ExtraSpace := 0;
  Invalidate;
end;

procedure TOfficeButton.CMMouseLeave(var Message: TMessage);
begin
  HasMouse := false;
  ExtraSpace := 0;
  Invalidate;
end;

procedure TOfficeButton.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  HasMouse := true;
  IsMouseDown := true;
  ExtraSpace := 1;
  Invalidate;
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TOfficeButton.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  IsMouseDown := false;
  ExtraSpace := 0;
  Invalidate;
  inherited MouseUp(Button, Shift, X, Y);
end;

procedure TOfficeButton.Paint;
var
  GlyphRect, BitmapRect: TRect;
  GlyphTop, GlyphLeft: integer;
begin
  GlyphTop := ((Height - FGlyph.Height) div 2) + ExtraSpace;
  GlyphLeft := 7 + ExtraSpace;
  GlyphRect := Rect(GlyphLeft, GlyphTop, FGlyph.Width+GlyphLeft, FGlyph.Height+GlyphTop);
  BitmapRect := Rect(0, 0, FGlyph.Width, FGlyph.Height);
  with Canvas do
  begin
    Brush.Style := bsClear;
    Brush.Color := Color;
    Font.Assign(FFont);

    if HasMouse then
    begin
      Pen.Color := BorderColor;
      Pen.Width := BorderWidth+1;
      RoundRect(BorderWidth,BorderWidth,Width-(BorderWidth-1),Height-(BorderWidth-1),FRoundSize,FRoundSize);

      Pen.Color := clWhite;
      Pen.Width := 2;
      MoveTo(FRoundSize div 2, BorderWidth+2);
      LineTo(Width - (FRoundSize div 2), BorderWidth+2);
    end
    else
    begin
      Pen.Color := clSilver;
      Pen.Width := BorderWidth;
      RoundRect(BorderWidth-1,BorderWidth-1,Width-(BorderWidth-1),Height-(BorderWidth-1),FRoundSize,FRoundSize);
    end;

    if HasMouse then
    begin
       if IsMouseDown then
          FGlyph.Handle := LoadBitmap(HInstance, 'DOWN')
       else
          FGlyph.Handle := LoadBitmap(HInstance, 'ON');
    end
    else
       FGlyph.Handle := LoadBitmap(HInstance, 'OFF');

    BrushCopy(GlyphRect, FGlyph, BitmapRect, clWhite);

    TextOut(((Width - TextWidth(FCaption)) div 2)+ GlyphLeft,((Height - TextHeight(FCaption)) div 2)+ExtraSpace,FCaption);
  end;
end;

constructor TOfficeButton.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FFont := TFont.Create;
  Width := 100;
  Height := 25;
  FCaption := 'OfficeButton';
  FGlyph := TBitmap.Create;
  FGlyph.Handle := LoadBitmap(HInstance, 'OFF');
  FRoundSize := 20;
  HasMouse := false;
  IsMouseDown := false;
  ExtraSpace := 0;
  FBorderColor := clGray;
  FBorderWidth := 1;
end;

destructor TOfficeButton.Destroy;
begin
  FFont.Free;
  FGlyph.Free;
  inherited Destroy;
end;

procedure TOfficeLabel.SetCaption(value: TCaption);
begin
  if value <> FCaption then
  begin
    FCaption := value;
    Invalidate;
  end;
end;

procedure TOfficeLabel.SetLEDType(value: TLEDType);
begin
  if value <> FLEDType then
  begin
    FLEDType := value;
    case FLEDType of
         RoundLED:   FGlyph.Handle := LoadBitmap(HInstance, 'OFFRND');
         TriLEDDown: FGlyph.Handle := LoadBitmap(HInstance, 'OFF3DWN');
         TriLEDUp:   FGlyph.Handle := LoadBitmap(HInstance, 'OFF3UP');
    end;
    Invalidate;
  end;
end;

procedure TOfficeLabel.SetFont(value: TFont);
begin
  FFont.Assign(value);
  Invalidate;
end;

procedure TOfficeLabel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  HasMouse := true;
  IsMouseDown := true;
  Invalidate;
  inherited MouseDown(Button, Shift, X, Y);
end;

procedure TOfficeLabel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  IsMouseDown := false;
  Invalidate;
  inherited MouseUp(Button, Shift, X, Y);
end;

procedure TOfficeLabel.CMMouseEnter(var Message: TMessage);
begin
  HasMouse := true;
  invalidate;
end;

procedure TOfficeLabel.CMMouseLeave(var Message: TMessage);
begin
  HasMouse := false;
  invalidate;
end;

procedure TOfficeLabel.Paint;
var
  GlyphRect, CaptionRect: TRect;
  CaptionLeft: word;
begin
  GlyphRect := Rect(0, 0, FGlyph.Width, FGlyph.Height);

  with Canvas do
  begin
    brush.style := bsClear;
    brush.color := Color;
    Font.Assign(FFont);

    if HasMouse then
    begin
       if IsMouseDown then
       begin
          case FLEDType of
               RoundLED:   FGlyph.Handle := LoadBitmap(HInstance, 'DOWNRND');
               TriLEDDown: FGlyph.Handle := LoadBitmap(HInstance, 'DOWN3DWN');
               TriLEDUp:   FGlyph.Handle := LoadBitmap(HInstance, 'DOWN3UP');
          end; // case statement
       end     // first option second if
       else
       begin
          case FLEDType of
               RoundLED:   FGlyph.Handle := LoadBitmap(HInstance, 'ONRND');
               TriLEDDown: FGlyph.Handle := LoadBitmap(HInstance, 'ON3DWN');
               TriLEDUp:   FGlyph.Handle := LoadBitmap(HInstance, 'ON3UP');
          end; // case statement
       end;    // second option second if
    end        // first option first if
    else
    begin
       case FLEDType of
            RoundLED:   FGlyph.Handle := LoadBitmap(HInstance, 'OFFRND');
            TriLEDDown: FGlyph.Handle := LoadBitmap(HInstance, 'OFF3DWN');
            TriLEDUp:   FGlyph.Handle := LoadBitmap(HInstance, 'OFF3UP');
       end;    // case statement
    end;       // second option first if

    BrushCopy(GlyphRect, FGlyph, GlyphRect, clYellow);

    CaptionLeft := FGlyph.Width + 5; // 5 is the space between the Glyph <-> Caption
    CaptionRect := Rect(CaptionLeft, 0, ClientWidth, ClientHeight);
    DrawText(Handle, PChar(FCaption), Length(FCaption), CaptionRect, DT_WORDBREAK or DT_NOPREFIX);

    if (HasMouse) and (IsMouseDown) then
    begin
      Brush.Style := bsClear;
      Brush.Color := Color;
      Pen.Color := clBlack;
      Pen.Width := 1;
      Pen.Style := psDot;
      Pen.Mode := pmMask;
      Rectangle(CaptionRect.Left-3, CaptionRect.Top, CaptionRect.Right-1, CaptionRect.Bottom-1);
    end;
  end;
end;

constructor TOfficeLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  width := 100;
  height := 16;
  FFont := TFont.Create;
  FCaption := 'OfficeLabel';
  LEDType := RoundLED;
  FGlyph := TBitmap.Create;
  FGlyph.Handle := LoadBitmap(HInstance, 'OFFRND');
end;

destructor TOfficeLabel.Destroy;
begin
  FFont.Free;
  FGlyph.Free;
  inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Win95', [TOfficeButton, TOfficeLabel]);
end;

end.
